<?php
/**
 * 接口应用_基于UniAPP开发的APP，涉及到的一些专用信息（如设备信息获取等）
 * @作者 phpok.com <admin@phpok.com>
 * @主页 www.phpok.com
 * @版本 5.x
 * @许可 www.phpok.com/lgpl.html PHPOK开源授权协议：GNU Lesser General Public License
 * @时间 2022年01月17日 11时57分
**/
namespace phpok\app\control\uniapp;
/**
 * 安全限制，防止直接访问
**/
if(!defined("PHPOK_SET")){
	exit("<h1>Access Denied</h1>");
}
class api_control extends \phpok_control
{
	public function __construct()
	{
		parent::control();
	}

	public function index_f()
	{
		$this->success();
	}

	/**
	 * 登记手机设备信息
	**/
	public function device_add_f()
	{
		$code = $this->get('code');
		if(!$code){
			$this->error('获取设备信息失败');
		}
		$sysinfo = $this->get("sysinfo","html");
		$sysinfo = stripslashes($sysinfo);
		$sysinfo = $this->lib('json')->decode($sysinfo);
		if(!$sysinfo){
			$this->error('暂无内容');
		}
		$user_id = 0;
		if($this->session->val('user_id')){
			$user_id = $this->session->val('user_id');
		}
		$tlist = array();
		$check = $this->model('uniapp')->device_info($code);
		if($check){
			$this->success();
		}
		//登记新的设备码
		$data = array();
		$data['site_id'] = $this->site['id'];
		$data['brand'] = $sysinfo['brand'];
		$data['xinghao'] = $sysinfo['model'];
		$data['client_platform'] = $sysinfo['platform'];
		$data['osversion'] = $sysinfo['system'];
		$data['lang'] = $sysinfo['language'];
		$data['version2'] = $sysinfo['version'];
		$data['screen_width'] = $sysinfo['screenWidth'];
		$data['screen_height'] = $sysinfo['screenHeight'];
		$data['body_height'] = $sysinfo['windowHeight'];
		$data['top_position'] = $sysinfo['windowTop'];
		$data['bottom_position'] = $sysinfo['windowBottom'];
		$data['statusbar_height'] = $sysinfo['statusBarHeight'];
		$data['dpi'] = $sysinfo['pixelRatio'];
		$data['device_code'] = $code; 
		$data['user_id'] = $user_id;
		$data['regtime'] = $this->time;
		$insert_id = $this->model('uniapp')->device_save($data);
		$this->success();
	}

	public function device_log_f()
	{
		$code = $this->get('code');
		if(!$code){
			$this->error('未找到设备信息，忽略日志');
		}
		$atime = $this->get('atime');
		if(!$atime){
			$this->error('日志信息不完整');
		}
		$etime = $this->get('etime');
		if(!$etime){
			$etime = $this->time;
		}
		$device = $this->model('uniapp')->device_info($code);
		if(!$device){
			$this->error('设备信息不存在');
		}
		$data = array();
		$data['device_id'] = $device['id'];
		$data['user_id'] = $this->session->val('user_id');
		$data['atime'] = $atime;
		$data['etime'] = $etime;
		$data['utimes'] = $etime > $atime ? ($etime - $atime) : 0;
		$this->model('uniapp')->log_save($data);
		$this->success();
	}

	/**
	 * 生成密钥
	**/
	public function keyval_f()
	{
		$data = $this->get('data','html');
		if(!$data){
			$this->error(P_Lang('未找到参数数据'));
		}
		$config = $this->model('config')->get_all();
		if(!$config || !$config['private_key'] || !$config['public_key']){
			$this->error(P_Lang('系统配置不全，请系统管理员到后台配置好公钥，密钥及API验证串'));
		}
		//解密
		$this->lib("token")->etype("public_key");
		$this->lib('token')->public_key($config['public_key']);
		$this->lib('token')->private_key($config['private_key']);
		$this->lib('token')->expiry(365*24*60*60);
		$keyid = $this->lib('token')->decode($data);
		if(!$keyid){
			$this->error(P_Lang('密钥为空'));
		}
		$rand = $this->lib('common')->str_rand(32,'all');
		$this->lib("token")->etype("api_code");
		$this->lib('token')->keyid($keyid);
		$this->lib('token')->expiry(365*24*60*60);
		$expiry = $this->time + 30*24*60*60;
		// 将客户端密钥存到 session 里
		$cache_id = $this->cache->id($keyid);
		$this->lib('file')->vi($rand,$this->dir_data.'uniapp/'.$cache_id.'.php');
		$data = array("time"=>$expiry,"value"=>$rand);
		$info = $this->lib('token')->encode($data);
		$this->success($info);
	}

	public function siteinfo_f()
	{
		$id = $this->get('id');
		if(!$id){
			$id = 'title';
		}
		$data = array();
		$list = explode(",",$id);
		foreach($list as $key=>$value){
			if($this->site[$value]){
				$data[$value] = $this->site[$value];
			}
		}
		$this->success($data);
	}

	/**
	 * 省市县信息
	**/
	public function pca_f()
	{
		$pca_id = 19;
		$tmplist = $this->model('opt')->opt_all('group_id='.$pca_id);
		$rslist = array();
		$this->pca_format($rslist,$tmplist,0);
		$this->success($rslist);
	}

	/**
	 * 检查更新
	 * 参数 version 新的版本，未写完
	**/
	public function update_f()
	{
		$version = $this->get('version');
		$this->update_error();
	}

	/**
	 * 获取IP相关信息
	**/
	public function ip_f()
	{
		$ip = $this->get('ip');
		if(!$ip){
			$ip = $this->lib('common')->ip();
		}
		$config = $this->model('config')->get_all();
		if(!$config){
			$this->error('未配置远程接口');
		}
		if(!$config['ok_status'] || !$config['ok_appid'] || !$config['ok_appkey'] || !$config['ok_domain']){
			$this->error('远程接口配置不完整');
		}
		$this->lib('phpok')->app_id($config['ok_appid']);
		$this->lib('phpok')->app_key($config['ok_appkey']);
		$this->lib('phpok')->server_url('https://'.$config['ok_domain'].'/apix-24920');
		$info = $this->lib('phpok')->content(array('ip'=>$ip));
		if(!$info || !$info['status']){
			$tip = $info['info'] ? $info['info'] : '获取数据失败';
			$this->error($tip);
		}
		$data = $info['info'];
		$data['ip'] = $ip;
		$this->success($data);
	}

	private function pca_format(&$rslist,$list,$parent_id=0){
		foreach($list as $key=>$value){
			if($value['parent_id'] == $parent_id){
				$tmp = array();
				$tmp['id'] = $value['id'];
				$tmp['name'] = $value['title'];
				$tmp['children'] = array();
				$this->pca_format($tmp['children'],$list,$value['id']);
				$rslist[] = $tmp;
			}
		}
	}

	private function update_error()
	{
		$data = array('code'=>1,'msg'=>'no');
		$info = $this->lib('json')->encode($data);
		exit($info);
	}

	private function update_success($version,$url,$log,$force=0)
	{
		$data = array('code'=>0,'msg'=>'ok');
		$data['version'] = $version;
		$data['url'] = $url;
		$data['log'] = '新版本【'.$version.'】'."\n".$log;
		$data['force'] = $force ? 1 : 0;
		$info = $this->lib('json')->encode($data);
		exit($info);
	}

	// 判断2个版本号的大小，并返回较大的那个，比如 1.0.9  1.1.0 ，则返回 1.1.0
	// 传入由2个点分隔为3组的版本号，返回较大的那个
	// 1.0.3  1.2.9 等
	private function getbig($one,$two)
	{
		$onearr=explode('.',$one);
		$twoarr=explode('.',$two);
		if(intval($onearr[0]) !== intval($twoarr[0])){
			return intval($onearr[0])>intval($twoarr[0])?$one:$two;
		}

		if(intval($onearr[1]) !== intval($twoarr[1])){
			return intval($onearr[1])>intval($twoarr[1])?$one:$two;
		}
		if(intval($onearr[2]) !== intval($twoarr[2])){
			return intval($onearr[2])>intval($twoarr[2])?$one:$two;
		}
		return $one;
	}
}
